# Copyright (c) 2022-2023 by Phase Advanced Sensor Systems, Inc.
# All rights reserved.
from .lorentz import Lorentzian
from .xy_series import XYSeries


__all__ = ['Lorentzian',
           'XYSeries',
           ]



import numpy as np


class XYSeries:
    def __init__(self, X, Y):
        assert len(X) == len(Y)
        self.X = np.array(X)
        self.Y = np.array(Y)

    def get_avg_value(self, x0, x1):
        indices = (self.X >= x0) & (self.X <= x1)
        X       = self.X[indices]
        Y       = self.Y[indices]
        if len(X) == 0:
            return None
        if len(X) == 1:
            return Y[0]

        area_2  = 0
        for i in range(len(X) - 1):
            area_2 += (Y[i + 1] + Y[i]) * (X[i + 1] - X[i])

        return area_2 / (2 * (X[-1] - X[0]))

    def append(self, x, y):
        self.X = np.append(self.X, x)
        self.Y = np.append(self.Y, y)



# Copyright (c) 2021-2023 by Phase Advanced Sensor Systems, Inc.
# All rights reserved.
import threading
import argparse
import time
import cmath
import math
from enum import Enum

import glotlib

import xtalx.z_sensor
from xtalx.z_sensor.peak_tracker import Delegate

from . import z_common


def interp(v0, vN, N, t):
    return (vN*t + v0*(N-t)) / N


HISTORY_PLOTS = 5
BASE_COLOR     = (0x1F/255, 0x77/255, 0xB4/255)
HISTORY_COLORS = [(interp(BASE_COLOR[0], 1, HISTORY_PLOTS, t),
                   interp(BASE_COLOR[1], 1, HISTORY_PLOTS, t),
                   interp(BASE_COLOR[2], 1, HISTORY_PLOTS, t),
                   1)
                  for t in range(HISTORY_PLOTS)]


LINE_WIDTH = 1


class TrackMode(Enum):
    CHIRP   = 1
    SWEEP   = 2


class ViewMode(Enum):
    LARGE   = 1
    SMALL   = 2


class TrackerWindow(glotlib.Window, Delegate):
    def __init__(self, tc, ipq, z_args, z_logger, name):
        super().__init__(900, 700, msaa=4, name=tc.serial_num or '')

        self.tc               = tc
        self.ipq              = ipq
        self.z_args           = z_args
        self.z_logger         = z_logger
        self.data_gen         = -1
        self.plot_gen         = -1
        self.data_lock        = threading.Lock()
        self.sweeps           = [[] for _ in range(HISTORY_PLOTS)]
        self.f_centers        = []
        self.widths           = []
        self.strengths        = []
        self.temperatures     = []
        self.densities        = []
        self.viscosities      = []
        self.tags             = []
        self.chirp_X_data     = None
        self.chirp_Y_data     = None
        self.chirp_X_lorentz  = None
        self.chirp_Y_lorentz  = None
        self.chirp_fit        = None
        self.chirp_freq       = None
        self.chirp_width      = None
        self.chirp_strength   = None
        self.sweep_snap       = False
        self.track_mode       = None
        self.view_mode        = ViewMode.LARGE
        self.large_w          = self.w_w
        self.large_h          = self.w_h
        self.small_w          = round(self.w_w * 0.75)
        self.small_h          = round(self.w_h * 0.25)
        self.sweep_prefix     = ''
        self.end_sweep_time   = None

        volts = tc.dac_to_a(z_args.amplitude)
        if volts:
            self.drive_str = 'Drive: %.2fV' % volts
        else:
            self.drive_str = 'Drive: %u DAC' % z_args.amplitude

        self.d_plot = self.add_plot(
            511, limits=(-0.1, -0.05, 50, 1.05), max_v_ticks=4)
        self.d_lines = self.d_plot.add_lines([], width=LINE_WIDTH)

        self.v_plot = self.add_plot(
            512, limits=(-0.1, -0.5, 50, 30.5), max_v_ticks=4,
            sharex=self.d_plot)
        self.v_lines = self.v_plot.add_lines([], width=LINE_WIDTH)

        t_lim = (262000, 262600) if tc.ginfo.have_temp_cal() else (-40, 200)
        self.t_plot = self.add_plot(
            513, limits=(-0.1, t_lim[0], 50, t_lim[1]), max_v_ticks=4,
            sharex=self.d_plot)
        self.t_lines = self.t_plot.add_lines([], width=LINE_WIDTH)

        self.fc_plot = self.add_plot(
            (5, 8, (25, 27)),
            limits=(-0.1, z_args.f_min, 50, z_args.f_max),
            max_v_ticks=4, sharex=self.d_plot)
        self.fc_lines = self.fc_plot.add_lines([], width=LINE_WIDTH)

        self.w_plot = self.add_plot(
            (5, 8, (28, 30)), limits=(-0.1, 0, 50, 100), max_v_ticks=4,
            sharex=self.d_plot)
        self.w_lines = self.w_plot.add_lines([], width=LINE_WIDTH)

        self.zx_plot = self.add_plot(
            (5, 4, 17), limits=(z_args.f_min, -1, z_args.f_max, 1),
            max_h_ticks=4, max_v_ticks=4)
        self.zx_lines = self._make_lines(self.zx_plot)

        self.phi_plot = self.add_plot(
            (5, 4, 18),
            limits=(z_args.f_min, -math.pi, z_args.f_max, math.pi),
            max_h_ticks=4, max_v_ticks=4, sharex=self.zx_plot)
        self.phi_lines = self._make_lines(self.phi_plot)

        self.rzx_plot = self.add_plot(
            (5, 4, 19), limits=(z_args.f_min, -1, z_args.f_max, 1),
            max_h_ticks=4, max_v_ticks=4, sharex=self.zx_plot)
        self.rzx_lines = self._make_lines(self.rzx_plot)
        self.rzx_lines[0].point_width = 3

        self.chirp_plot = self.add_plot(
            (5, 4, (17, 19)), limits=(15000, 0, 40000, 0.01),
            max_h_ticks=8, max_v_ticks=4, visible=False)
        self.chirp_line = self.chirp_plot.add_steps()
        self.chirp_lorentz_line = self.chirp_plot.add_lines()

        self.nyq_plot = self.add_plot(
            (5, 4, (16, 20)), max_h_ticks=4, max_v_ticks=4,
            aspect=glotlib.ASPECT_SQUARE)
        self.nyq_plot.add_hline(0, color=(0.5, 0.75, 0.5))
        self.nyq_plot.add_vline(0, color=(0.5, 0.75, 0.5))
        self.nyq_lines = self._make_lines(self.nyq_plot)
        self.nyq_lines[0].point_width = 3

        self.status_text = self.add_label((0.01, 0.01), self.drive_str)
        self.pos_label   = self.add_label((0.99, 0.01), '', anchor='SE')
        self.freq_label  = self.add_label((0.01, 0.99), '', anchor='NW')
        self.str_label   = self.add_label((0.50, 0.99), '', anchor='N')
        self.width_label = self.add_label((0.99, 0.99), '', anchor='NE')

        glotlib.periodic(1, self.update_periodic)

    @staticmethod
    def _make_lines(plot):
        lines = [plot.add_lines([], color=HISTORY_COLORS[-(i+1)],
                                width=LINE_WIDTH)
                 for i in range(HISTORY_PLOTS)]
        lines.reverse()
        return lines

    def update_periodic(self, _t):
        self.mark_dirty()

    def handle_window_size_changed(self):
        if self.view_mode == ViewMode.LARGE:
            self.large_w, self.large_h = self.w_w, self.w_h
        elif self.view_mode == ViewMode.SMALL:
            self.small_w, self.small_h = self.w_w, self.w_h

    def handle_mouse_moved(self, x, y):
        self.mark_dirty()

    def handle_key_press(self, key):
        if key == glotlib.KEY_ESCAPE:
            if self.view_mode == ViewMode.LARGE:
                self.shrink_window()
            elif self.view_mode == ViewMode.SMALL:
                self.expand_window()

    def shrink_window(self):
        if self.view_mode == ViewMode.SMALL:
            return

        self.d_plot.hide()
        self.v_plot.hide()
        self.t_plot.hide()
        self.fc_plot.hide()
        self.w_plot.hide()
        self.zx_plot.set_bounds(141, pad_b=0.2, pad_t=0.1)
        self.phi_plot.set_bounds(142, pad_b=0.2, pad_t=0.1)
        self.rzx_plot.set_bounds(143, pad_b=0.2, pad_t=0.1)
        self.chirp_plot.set_bounds((1, 4, (1, 3)), pad_b=0.2, pad_t=0.1)
        self.nyq_plot.set_bounds(144, pad_b=0.2, pad_t=0.1)
        self.view_mode = ViewMode.SMALL
        self.resize(self.small_w, self.small_h)

    def expand_window(self):
        if self.view_mode == ViewMode.LARGE:
            return

        self.d_plot.show()
        self.v_plot.show()
        self.t_plot.show()
        self.fc_plot.show()
        self.w_plot.show()
        self.zx_plot.set_bounds((5, 4, 17))
        self.phi_plot.set_bounds((5, 4, 18))
        self.rzx_plot.set_bounds((5, 4, 19))
        self.chirp_plot.set_bounds((5, 4, (17, 19)))
        self.nyq_plot.set_bounds((5, 4, (16, 20)))
        self.view_mode = ViewMode.LARGE
        self.resize(self.large_w, self.large_h)

    def set_status_text(self, t):
        return self.status_text.set_text('%s\n%s' % (self.drive_str, t))

    def update_geometry(self, _t):
        if self.track_mode == TrackMode.SWEEP:
            return self.update_sweep_geometry()
        if self.track_mode == TrackMode.CHIRP:
            return self.update_chirp_geometry()
        return False

    def update_sweep_geometry(self):
        if self.end_sweep_time is not None:
            dt = max(self.end_sweep_time - time.time(), 0)
            updated = self.set_status_text('%s: %.0f seconds' %
                                           (self.sweep_prefix, dt))
        else:
            updated = self.set_status_text('')

        _, _, _, data_x, data_y = self.get_mouse_pos()
        if data_x is not None:
            updated |= self.pos_label.set_text('%.10f  %.10f' %
                                               (data_x, data_y))

        with self.data_lock:
            if self.data_gen == self.plot_gen:
                return updated
            self.plot_gen  = self.data_gen
            sweeps         = self.sweeps[:]
            f_centers      = self.f_centers[:]
            widths         = self.widths[:]
            strengths      = self.strengths[:]
            temperatures   = self.temperatures[:]
            densities      = self.densities[:]
            viscosities    = self.viscosities[:]
            sweep_snap     = self.sweep_snap
            # tags           = self.tags[:]

            self.sweep_snap = False

        ts = list(range(len(f_centers)))
        self.fc_lines.set_x_y_data(ts, f_centers)
        self.d_lines.set_x_y_data(ts, densities)
        self.v_lines.set_x_y_data(ts, viscosities)
        self.t_lines.set_x_y_data(ts, temperatures)
        self.w_lines.set_x_y_data(ts, widths)

        for i, sweep in enumerate(sweeps):
            fs = [p.f for p in sweep]
            self.zx_lines[i].set_x_y_data(fs, [abs(p.z) for p in sweep])
            self.phi_lines[i].set_x_y_data(fs,
                                           [cmath.phase(p.z) for p in sweep])
            self.rzx_lines[i].set_x_y_data(fs, [p.z.real for p in sweep])
            self.nyq_lines[i].set_x_y_data([p.z.real for p in sweep],
                                           [-p.z.imag for p in sweep])

        freq = f_centers[-1]
        if self.chirp_freq is not None:
            self.freq_label.set_text('F: %.3f Hz' % self.chirp_freq)
        elif freq is not None:
            self.freq_label.set_text('F: %.3f Hz' % freq)
        else:
            self.freq_label.set_text('')

        strength = strengths[-1]
        if self.chirp_strength is not None:
            self.str_label.set_text('S: %.2f' % self.chirp_strength)
        elif strength is not None:
            self.str_label.set_text('S: %.3e' % strength)
        else:
            self.str_label.set_text('')

        width = widths[-1]
        if self.chirp_width is not None:
            self.width_label.set_text('W: %.3f Hz' % self.chirp_width)
        elif width is not None:
            self.width_label.set_text('W: %.3f Hz' % width)
        else:
            self.width_label.set_text('')

        if sweep_snap:
            self.zx_plot.snap_bounds()
            self.rzx_plot.snap_bounds()
            self.nyq_plot.snap_bounds()

        return True

    def update_chirp_geometry(self):
        with self.data_lock:
            if self.chirp_X_data is None:
                return False

            chirp_X_data         = self.chirp_X_data
            chirp_Y_data         = self.chirp_Y_data
            chirp_X_lorentz      = self.chirp_X_lorentz
            chirp_Y_lorentz      = self.chirp_Y_lorentz
            chirp_fit            = self.chirp_fit
            self.chirp_X_data    = None
            self.chirp_Y_data    = None
            self.chirp_X_lorentz = None
            self.chirp_Y_lorentz = None
            self.chirp_fit       = None

        self.chirp_line.set_x_y_data(chirp_X_data, chirp_Y_data)
        self.chirp_lorentz_line.set_x_y_data(chirp_X_lorentz, chirp_Y_lorentz)
        self.chirp_plot.snap_bounds()

        if self.chirp_freq is not None:
            self.freq_label.set_text('F: %.3f Hz' % self.chirp_freq)
        if self.chirp_width is not None:
            self.width_label.set_text('W: %.3f Hz' % self.chirp_width)
        if self.chirp_strength is not None:
            self.str_label.set_text('S: %.2f' % self.chirp_strength)

        if chirp_fit:
            self.set_status_text('Chirping: R**2 = %.5f' % chirp_fit.RR)
        else:
            self.set_status_text('Chirping: (No fit)')

        return True

    def data_callback(self, _sweep, fw_fit, points, T, D, V, hires):
        with self.data_lock:
            self.sweeps.pop()
            self.sweeps.insert(0, points)
            if fw_fit is not None:
                self.f_centers.append(fw_fit.peak_hz)
                self.widths.append(fw_fit.peak_fwhm)
                self.strengths.append(fw_fit.strength)
            else:
                self.f_centers.append(None)
                self.widths.append(None)
                self.strengths.append(None)
            self.temperatures.append(T)
            self.densities.append(D)
            self.viscosities.append(V)
            self.tags.append('Hires' if hires else 'Search')
            self.data_gen += 1

            self.chirp_plot.hide()
            self.zx_plot.show()
            self.phi_plot.show()
            self.rzx_plot.show()

            self.chirp_freq     = None
            self.chirp_width    = None
            self.chirp_strength = None

            self.mark_dirty()

    def chirp_callback(self, _tc, _pt, _n, lf, X_data, Y_data, X_lorentzian,
                       Y_lorentzian):
        with self.data_lock:
            self.track_mode = TrackMode.CHIRP
            self.sweep_snap = True

            self.chirp_plot.show()
            self.zx_plot.hide()
            self.phi_plot.hide()
            self.rzx_plot.hide()

            self.chirp_X_data    = X_data
            self.chirp_Y_data    = Y_data
            self.chirp_X_lorentz = X_lorentzian
            self.chirp_Y_lorentz = Y_lorentzian
            self.chirp_fit       = lf
            if lf is not None:
                self.chirp_freq     = lf.x0
                self.chirp_width    = 2 * lf.W
                self.chirp_strength = lf.A / (math.pi * lf.W)
            else:
                self.chirp_freq     = None
                self.chirp_width    = None
                self.chirp_strength = None

            self.mark_dirty()

    def sweep_started_callback(self, _tc, pt, _duration_ms, hires, f0, f1):
        with self.data_lock:
            self.track_mode     = TrackMode.SWEEP
            self.end_sweep_time = pt.t_timeout
            freq_str            = '[%.2f - %.2f]' % (f0, f1)
            if hires:
                self.sweep_prefix = 'Sweeping Hires %s' % freq_str
            else:
                self.sweep_prefix = 'Searching %s' % freq_str
            self.mark_dirty()

    def sweep_callback(self, tc, pt, t0_ns, duration_ms, points, fw_fit, hires,
                       temp_freq):
        T, D, V = self.z_logger.log_sweep(tc, pt, t0_ns, points, fw_fit, hires,
                                          temp_freq)
        self.data_callback(pt.sweep, fw_fit, points, T, D, V, hires)

        if self.ipq:
            p = self.make_influx_point(t0_ns, duration_ms, fw_fit, hires,
                                       temp_freq)
            self.ipq.append(p)

    def make_influx_point(self, t0_ns, duration_ms, fw_fit, hires, temp_freq):
        return self.tc.make_influx_point(t0_ns, duration_ms, fw_fit, hires,
                                         temp_freq)


def main(rv):
    _c, ipq = z_common.parse_config(rv)

    track_admittance = not rv.track_impedance

    dev    = xtalx.z_sensor.find_one(serial_number=rv.sensor)
    tc     = xtalx.z_sensor.make(dev, verbose=rv.verbose,
                                 yield_Y=track_admittance)
    za, zl = z_common.parse_args(tc, rv)
    tw     = TrackerWindow(tc, ipq, za, zl, tc.serial_num)
    pt     = xtalx.z_sensor.PeakTracker(
              tc, za.amplitude, za.f0, za.f1, za.df, za.nfreqs,
              za.search_time_secs, za.sweep_time_secs,
              settle_ms=za.settle_ms, delegate=tw)
    pt.start_threaded()

    try:
        glotlib.interact()
    except KeyboardInterrupt:
        print()
    finally:
        pt.stop_threaded()

    if ipq:
        ipq.flush()


def _main():
    parser = argparse.ArgumentParser()
    z_common.add_arguments(parser)
    rv = parser.parse_args()
    main(rv)


if __name__ == '__main__':
    _main()



# Copyright (c) 2021-2023 by Phase Advanced Sensor Systems, Inc.
# All rights reserved.
import time


class TinCan:
    def __init__(self, verbose=False, yield_Y=False):
        self.verbose         = verbose
        self.yield_Y         = yield_Y
        self.serial_num      = None

    def log(self, tag, s, timestamp=None):
        timestamp = timestamp or time.time_ns()
        print('[%u - %s] %s: %s' % (timestamp, self.serial_num, tag, s))

    def info(self, s, **kwargs):
        self.log('I', s, **kwargs)

    def warn(self, s, **kwargs):
        self.log('W', s, **kwargs)

    def vlog(self, tag, s, **kwargs):
        if self.verbose:
            self.log(tag, s, **kwargs)

    def dbg(self, s, **kwargs):
        self.vlog('D', s, **kwargs)

    def make_influx_point(self, t0_ns, duration_ms, fw_fit, hires, temp_freq):
        p = {
            'measurement' : 'tincan_data',
            'time'        : t0_ns,
            'tags'        : {'sensor' : self.serial_num},
            'fields'      :
            {
                'hires'       : hires,
                'duration_ms' : duration_ms,
            }
        }

        if temp_freq is not None:
            p['fields']['temp_freq_hz'] = float(temp_freq)

        if fw_fit is None:
            return p

        p['fields'].update({
            'peak_hz'   : float(fw_fit.peak_hz),
            'peak_fwhm' : float(fw_fit.peak_fwhm),
            'RR'        : float(fw_fit.RR),
        })

        if fw_fit.temp_c is not None:
            p['fields']['temp_c'] = float(fw_fit.temp_c)
        if fw_fit.density_g_per_ml is not None:
            p['fields']['density_g_per_ml'] = float(fw_fit.density_g_per_ml)
        if fw_fit.viscosity_cp is not None:
            p['fields']['viscosity_cp'] = float(fw_fit.viscosity_cp)

        return p



# Copyright (c) 2021-2023 by Phase Advanced Sensor Systems, Inc.
# All rights reserved.

class SignalInfo:
    def __init__(self, X, Y, clk, name, phasor=None, offset=None, RR=None):
        self.phasor  = phasor
        self.offset  = offset
        self.RR      = RR
        self.X       = X
        self.Y       = Y
        self.clk     = clk
        self.name    = name


class ScopeData:
    def __init__(self, freq, w, hdr=None, data=None):
        self.freq     = freq
        self.w        = w
        self.hdr      = hdr
        self.data     = data
        self.sig_info = []
        self.map      = {}

    def __getitem__(self, key):
        return self.map[key]

    def add_signal(self, X, Y, clk, name, **kwargs):
        self.sig_info.append(SignalInfo(X, Y, clk, name, **kwargs))
        if name:
            self.map[name] = self.sig_info[-1]



# Copyright (c) 2021-2023 by Phase Advanced Sensor Systems, Inc.
# All rights reserved.
import threading
import time
import math
import enum

import numpy as np

from xtalx.tools.math import Lorentzian


CHIRP_F0     = 15000
CHIRP_F1     = 35000
CHIRP_A      = 0.25
CHIRP_MS     = 105
CHIRP_DT     = CHIRP_MS * 0.001 * 2 + 0.02
CHIRP_MIN_RR = 0.2


def is_good_freq(f):
    return 10000 <= f <= 45000


class State(enum.Enum):
    # Waiting for some command.
    IDLE = 0

    # We have started a chirp to try and estimate the search parameters.
    CHIRP_WAIT_DATA = 1

    # We have started a low-resolution sweep to try and estimate the center
    # frequency and width.
    PEAK_SEARCH_WAIT_DATA = 2

    # We have started a hi-resolution sweep to get data for computing the
    # density and viscosity.
    HIRES_SWEEP_WAIT_DATA = 3


class Delegate:
    def chirp_callback(self, tc, pt, n, lf, X_data, Y_data, X_lorentzian,
                       Y_lorentzian):
        pass

    def sweep_started_callback(self, tc, pt, duration_ms, hires, f0, f1):
        pass

    def sweep_callback(self, tc, pt, t0_ns, duration_ms, points, fw_fit, hires,
                       temp_freq):
        pass


class PeakTracker:
    def __init__(self, tc, amplitude, f0, f1, search_df, nfreqs, search_time,
                 sweep_time, settle_ms=2, delegate=Delegate()):
        self.tc             = tc
        self.amplitude      = amplitude
        self.f0             = f0
        self.f1             = f1
        self.search_df      = search_df
        self.nfreqs         = nfreqs
        self.search_min_f   = min(f0, f1)
        self.search_max_f   = max(f0, f1)
        self.search_time    = search_time
        self.sweep_time     = sweep_time
        self.settle_ms      = settle_ms
        self.delegate       = delegate
        self.thread         = None
        self.thread_cond    = threading.Condition()
        self.thread_exc     = None

        self.t_timeout      = None
        self.hires_f_center = None
        self.hires_width    = None
        self.min_rr         = None
        self.min_w          = None
        self.sensor_ms      = None
        self.sweep          = 0
        self.sweep_t0_ns    = None
        self.state          = State.IDLE
        self.chirp_space    = np.linspace(CHIRP_F0, CHIRP_F1,
                                          int(CHIRP_F1 - CHIRP_F0))

    def _transition(self, new_state):
        if self.state == new_state:
            return

        self.tc.dbg('Transition: %s -> %s' % (self.state, new_state))
        self.state = new_state

    def _start_sweep(self, ftups, hires):
        self.sweep_t0_ns   = time.time_ns()
        dt, self.sensor_ms = self.tc.sweep_async(self.amplitude, ftups,
                                                 ndiscards=self.settle_ms)
        self.t_timeout = time.time() + dt
        self.delegate.sweep_started_callback(self.tc, self, self.sensor_ms,
                                             hires, ftups[0][0], ftups[-1][0])

    def _read_sweep_points(self):
        points = self.tc.read_sweep_data().results
        max_amplitude = max(p.amplitude[1] for p in points)
        if max_amplitude >= self.tc.ADC_MAX / 2:
            self.tc.warn('Possible amplitude clipping.  Max amplitude '
                         'of %s exceeds maximum of %u.'
                         % (max_amplitude, self.tc.ADC_MAX // 2))

        return points

    def _get_sweep_fit(self, temp_hz):
        fit = self.tc.get_sweep_fit(temp_hz)
        if not 1 <= fit.status <= 4:
            self.tc.warn('FwFit failed: %s s status %d niter %d' %
                         (fit.dt / 1e9, fit.status, fit.niter))
            fit = None

        return fit

    def _start_full_search(self):
        self._start_chirp()

    def _start_chirp(self):
        '''
        Start chirping and refine the data as it comes in.
        '''
        self.tc.info('Chirping from %f to %f...' % (CHIRP_F0, CHIRP_F1))
        self.tc.send_auto_chirp_cmd(CHIRP_F0, CHIRP_F1,
                                    round(self.tc.a_to_dac(CHIRP_A)))
        self._transition(State.CHIRP_WAIT_DATA)
        self.t_timeout = time.time() + CHIRP_DT

    def _start_peak_search(self, min_f, max_f, df):
        '''
        Do a fast sweep of the full frequency range to try and find the peak.
        '''
        min_f       = math.floor(min_f / df) * df
        max_f       = math.ceil(max_f / df) * df
        N           = int(1 + (max_f - min_f) // df)
        dt          = math.ceil(1000 * self.search_time / N)
        freqs       = [min_f + i*df for i in range(N)]
        ftups       = [(f, dt) for f in freqs if is_good_freq(f)]
        self.min_rr = 0.5
        self.min_w  = 12
        self._start_sweep(ftups, False)
        self._transition(State.PEAK_SEARCH_WAIT_DATA)

    def _start_peak_search_defaults(self):
        self._start_peak_search(self.search_min_f, self.search_max_f,
                                self.search_df)

    def _start_hires_sweep(self):
        '''
        Do a high-resolution sweep centered on the last peak we found.
        '''
        freqs       = self.tc.gen_hires_freqs(self.hires_f_center,
                                              self.hires_width,
                                              self.nfreqs // 2)
        dt          = math.ceil(1000 * self.sweep_time / len(freqs))
        ftups       = [(f, dt) for f in freqs if is_good_freq(f)]
        self.min_rr = 0.81
        self.min_w  = 0
        self._start_sweep(ftups, True)
        self._transition(State.HIRES_SWEEP_WAIT_DATA)

    def _get_temp_freq(self):
        self.tc.set_t_enable(True)
        time.sleep(0.5)
        t0_crystal_ticks, t0_cpu_ticks = self.tc.read_temp()
        time.sleep(0.5)
        t1_crystal_ticks, t1_cpu_ticks = self.tc.read_temp()
        self.tc.set_t_enable(False)
        time.sleep(0.5)

        dt = (t1_cpu_ticks - t0_cpu_ticks) / self.tc.CPU_FREQ
        if dt == 0:
            return None

        dcrystal = (t1_crystal_ticks - t0_crystal_ticks) & 0xFFFFFFFF
        return dcrystal * 8 / dt

    def _handle_timeout(self):
        if self.state in (State.PEAK_SEARCH_WAIT_DATA,
                          State.HIRES_SWEEP_WAIT_DATA):
            self._handle_sweep_timeout()
        elif self.state == State.CHIRP_WAIT_DATA:
            self._handle_chirp_timeout()

    def _handle_sweep_timeout(self):
        points = self._read_sweep_points()
        points = points[1:]

        t0_ns = time.time_ns()

        temp_freq = self._get_temp_freq()
        fw_fit    = self._get_sweep_fit(temp_freq)
        hires     = (self.state == State.HIRES_SWEEP_WAIT_DATA)

        t1_ns = time.time_ns()
        dt_ms = round((t1_ns - t0_ns) / 1000000)

        self.delegate.sweep_callback(self.tc, self, self.sweep_t0_ns,
                                     self.sensor_ms + dt_ms, points, fw_fit,
                                     hires, temp_freq)
        self.sweep += 1

        if self.state == State.IDLE:
            return

        if fw_fit is None:
            self.tc.warn('Fit failed due to possible transient, repeating '
                         'peak search.')
            self._start_full_search()
            return

        if fw_fit.RR < self.min_rr:
            self.tc.warn('Confidence too low, repeating peak search.')
            self._start_full_search()
            return

        if not 15000 <= fw_fit.peak_hz <= 35000:
            self.tc.warn('Detected out-of-bounds peak, repeating peak search.')
            self._start_full_search()
            return

        self.hires_width    = max(abs(fw_fit.peak_fwhm), self.min_w)
        self.hires_f_center = fw_fit.peak_hz
        self._start_hires_sweep()

    def _handle_chirp_timeout(self):
        res = self.tc.sample_auto_chirp_sync()
        self.t_timeout = time.time() + CHIRP_DT

        X = np.linspace(res.f0, res.f1, res.nbins)
        Y = res.bins[res.bin0:res.bin1 + 1]
        try:
            lf = Lorentzian.from_x_y(X, Y, res.A, res.x0, res.W)
            Y2 = lf(self.chirp_space)
        except RuntimeError:
            lf = None
            Y2 = np.zeros(len(self.chirp_space))

        self.delegate.chirp_callback(self.tc, self, res.nchirps, lf, X, Y,
                                     self.chirp_space, Y2)

        if res.nchirps < 20:
            return

        self.tc._synchronize()

        if lf and lf.RR >= CHIRP_MIN_RR and CHIRP_F0 <= lf.x0 <= CHIRP_F1:
            strength = lf.A / (math.pi * lf.W)
            self.tc.info('Chirp succeeded: peak_hz %s peak_fwhm %s '
                         'strength %.2f RR %.5f' %
                         (lf.x0, lf.W*2, strength, lf.RR))
            self.hires_width    = max(abs(lf.W * 2), 12)
            self.hires_f_center = lf.x0
            self._start_hires_sweep()
            return

        if lf:
            self.tc.info('Chirp failed with RR = %.5f' % lf.RR)
        else:
            self.tc.info('Chirp failed with no fit.')
        self._start_peak_search_defaults()

    def start_async(self):
        assert self.state == State.IDLE
        self._start_full_search()

    def stop_async(self):
        assert self.state != State.IDLE
        self.t_timeout = None
        self._transition(State.IDLE)

    def poll(self):
        # Process any pending timeout.
        if self.t_timeout is not None:
            t = time.time()
            if t >= self.t_timeout:
                self._handle_timeout()

        # If a timeout is armed, return the time delta for when next to poll.
        if self.t_timeout is not None:
            return max(self.t_timeout - time.time(), 0)

        # Poll continuously.
        return 0

    def start_threaded(self):
        with self.thread_cond:
            assert self.thread is None
            self.thread_exc = None
            self.thread     = threading.Thread(target=self._poll_threaded)
            self.thread.start()

    def stop_threaded(self):
        with self.thread_cond:
            t, self.thread = self.thread, None
            self.thread_cond.notify()
        t.join()

    def _poll_threaded(self):
        try:
            with self.thread_cond:
                if self.thread:
                    self._start_full_search()
                    while self.thread:
                        dt = self.poll()
                        self.thread_cond.wait(timeout=dt)
        except Exception as e:
            self.thread_exc = e



# Copyright (c) 2020 by Phase Advanced Sensor Systems, Inc.
# All rights reserved.
import threading
import time
from influxdb import InfluxDBClient


class InfluxDBPushQueue:
    '''
    Class to asynchronously push data points to an InfluxDB instance.  Pushing
    points can take a nondeterministic length of time and by trying to push
    them synchronously you can introduce lots of jitter into your measurement.
    This asynchronous queue allows the work to be performed in a separate
    thread so as not to disturb the measurement times.
    '''
    def __init__(self, host, port, username, password, database=None,
                 push_cb=None, **kwargs):
        self.push_cb = push_cb
        self.idb     = InfluxDBClient(host, port, username, password, **kwargs)
        if database:
            self.idb.switch_database(database)

        self.queue_cond   = threading.Condition()
        self.queue        = []
        self.cookie_queue = []
        self.thread       = None
        self.running      = False
        self.start()

    def start(self):
        assert not self.thread
        self.running = True
        self.thread  = threading.Thread(target=self._push_loop, daemon=True)
        self.thread.start()

    def append(self, p, cookie=None):
        '''
        Append a single point to the push queue.
        '''
        with self.queue_cond:
            self.queue.append(p)
            self.cookie_queue.append(cookie)
            self.queue_cond.notify()

    def append_list(self, ps, cookies=None):
        '''
        Append a list of points to the push queue.
        '''
        with self.queue_cond:
            self.queue += ps
            if cookies is not None:
                self.cookie_queue += cookies
            else:
                self.cookie_queue += [None] * len(ps)
            self.queue_cond.notify()

    def flush(self):
        '''
        Flush the current queue elements.
        '''
        while self.queue:
            pass
        with self.queue_cond:
            self.running = False
            self.queue_cond.notify()
        self.thread.join()
        self.thread = None
        self.start()

    def _push_loop(self):
        while self.queue or self.running:
            with self.queue_cond:
                while not self.queue and self.running:
                    self.queue_cond.wait()

                points            = self.queue
                cookies           = self.cookie_queue
                self.queue        = []
                self.cookie_queue = []

            if points:
                while points:
                    push_points = points[:10000]
                    points      = points[10000:]
                    while True:
                        try:
                            self.idb.write_points(push_points,
                                                  time_precision='n')
                            break
                        except Exception as e:
                            print('InfluxDB push exception: %s' % e)
                            print('Retrying in 30 seconds...')
                            time.sleep(30)

                    if self.push_cb:
                        for p, c in zip(push_points, cookies):
                            self.push_cb(p, c)



# Copyright (c) 2023 by Phase Advanced Sensor Systems, Inc.
# All rights reserved.
import threading
import time
import enum

from .peak_tracker import Delegate


class State(enum.Enum):
    # Discarding measurements except for the most recent.
    IDLE = 0

    # Queueing measurements until a condition is satisfied.
    QUEUEING = 1

    # Predicate has matched.
    MATCHED = 2

    # Matching aborted.
    ABORTED = 3


class Measurement:
    def __init__(self, t0_ns, duration_ms, points, fw_fit, temp_freq):
        self.t0_ns            = t0_ns
        self.duration_ms      = duration_ms
        self.points           = points
        self.fw_fit           = fw_fit
        self.temp_freq        = temp_freq
        if fw_fit:
            self.peak_hz          = fw_fit.peak_hz
            self.peak_fwhm        = fw_fit.peak_fwhm
            self.density_g_per_ml = fw_fit.density_g_per_ml
            self.viscosity_cp     = fw_fit.viscosity_cp
        else:
            self.peak_hz          = None
            self.peak_fwhm        = None
            self.density_g_per_ml = None
            self.viscosity_cp     = None

    def __repr__(self):
        return ('Measurement(peak_hz=%f, peak_fwhm=%f)' %
                (self.peak_hz, self.peak_fwhm))


class PredicateQueue(Delegate):
    def __init__(self, delegate=None):
        self.queue_cond = threading.Condition()
        self.queue      = []
        self.state      = State.IDLE
        self.predicate  = None
        self.N          = None
        self.delegate   = delegate

    def chirp_callback(self, *args):
        if self.delegate:
            self.delegate.chirp_callback(*args)

    def sweep_callback(self, tc, pt, t0_ns, duration_ms, points, fw_fit, hires,
                       temp_freq):
        if self.delegate:
            self.delegate.sweep_callback(tc, pt, t0_ns, duration_ms, points,
                                         fw_fit, hires, temp_freq)

        m = Measurement(t0_ns, duration_ms, points, fw_fit, temp_freq)

        with self.queue_cond:
            if not hires:
                self.queue.clear()
                return

            self.queue.append(m)

            if self.state in (State.IDLE, State.MATCHED):
                return

            if self.state == State.QUEUEING:
                if len(self.queue) >= self.N:
                    if self.predicate(
                            self.queue[-self.N:]):  # pylint: disable=E1130
                        self.state = State.MATCHED
                        self.queue_cond.notify()

    def _wait_predicate_locked(self, predicate, N, timeout):
        self.state     = State.QUEUEING
        self.predicate = predicate
        self.N         = N

        start_time = time.time()
        rem        = timeout
        while True:
            if self.state == State.MATCHED:
                return True
            if ((rem is not None and rem <= 0) or
                    self.state == State.ABORTED):
                self.state = State.IDLE
                return False

            if timeout:
                self.queue_cond.wait(timeout=rem)
                rem = start_time + timeout - time.time()
            else:
                self.queue_cond.wait()

    def wait_predicate(self, predicate, N, timeout=None):
        '''
        Wait until the specified predicate returns true.  The predicate is a
        callable which takes a single argument, measurements, a list of N
        measurements.  The callable should return True if the predicate
        condition is matched (for instance, the standard deviation of all the
        N measurements is below a critical value), or False otherwise.

        The queue is not cleared when wait_predicate() is invoked, however at
        least one measurement must come in before the call will return - i.e.
        if the predicate would currently match against the elements currently
        in the queue, it will still wait for at least one more measurement to
        be made before evaluating the predicate.

        When the predicate matches, a list of the N measurements used in the
        evaluation is returned.  If the predicate fails to match in the timeout
        period, None is returned.
        '''
        with self.queue_cond:
            if self._wait_predicate_locked(predicate, N, timeout):
                return self.queue[:]
        return None

    def get_measurements(self):
        '''
        Returns a list of all measurements made since the last time the queue
        was cleared.  The queue will clear automatically when a lo-res (search)
        measurement is made, so only usable measurements are returned.
        '''
        with self.queue_cond:
            return self.queue[:]

    def get_measurement(self):
        '''
        Returns the most recent measurement, or None if no measurement is
        available yet.
        '''
        with self.queue_cond:
            if not self.queue:
                return None
            return self.queue[0]

    def clear(self):
        '''
        Clears all measurements from the queue so that matching will only be
        performed on new measurements.
        '''
        with self.queue_cond:
            self.queue.clear()

    def abort(self):
        '''
        Abort any wait_predicate() call in progress.
        '''
        with self.queue_cond:
            self.state = State.ABORTED
            self.queue_cond.notify()



# Copyright (c) 2023 by Phase Advanced Sensor Systems, Inc.
# All rights reserved.
import threading
import argparse
import math
import time

import glotlib

import xtalx.p_sensor
from xtalx.tools.math import XYSeries


LINE_WIDTH = 1


class TrackerWindow(glotlib.Window):
    def __init__(self, name, period):
        super().__init__(900, 700, msaa=4, name=name)

        self.period         = period
        self.data_gen       = -1
        self.plot_gen       = -1
        self.data_lock      = threading.Lock()
        self.new_data       = []
        self.p_measurements = XYSeries([], [])
        self.t_measurements = XYSeries([], [])

        self.p_plot = self.add_plot(
            311, limits=(-0.1, -0.5, 120, 300), max_v_ticks=10)
        self.p_lines = self.p_plot.add_lines([], width=LINE_WIDTH)

        self.p_slow_plot = self.add_plot(
            312, limits=(-0.1, -0.5, 120, 300), max_v_ticks=10,
            sharex=self.p_plot, sharey=self.p_plot)
        self.p_slow_lines = self.p_slow_plot.add_lines([], width=LINE_WIDTH)

        self.t_plot = self.add_plot(
            313, limits=(-0.1, -0.5, 120, 50), max_v_ticks=10,
            sharex=self.p_plot)
        self.t_lines = self.t_plot.add_lines([], width=LINE_WIDTH)

        self.pos_label = self.add_label((0.99, 0.01), '', anchor='SE')

        y0 = (self.p_plot.bounds[1] + self.p_plot.bounds[3]) / 2
        y1 = (self.p_slow_plot.bounds[1] + self.p_slow_plot.bounds[3]) / 2
        y2 = (self.t_plot.bounds[1] + self.t_plot.bounds[3]) / 2
        self.add_label((.035, y0), 'PSI', anchor='NW', theta=math.pi/2)
        self.add_label((.035, y1), 'PSI (%u-sec Avg)' % period, anchor='NW',
                       theta=math.pi/2)
        self.add_label((.035, y2), 'Temp (C)', anchor='NW', theta=math.pi/2)

        self.mouse_vlines = [self.p_plot.add_vline(0, color='#80C080'),
                             self.p_slow_plot.add_vline(0, color='#80C080'),
                             self.t_plot.add_vline(0, color='#80C080')]

    def handle_mouse_moved(self, x, y):
        data_x, _ = self.p_plot._window_to_data(x, y)
        for vline in self.mouse_vlines:
            vline.set_x_data(data_x)
        self.mark_dirty()

    def update_geometry(self, _t):
        updated = False

        _, _, _, data_x, data_y = self.get_mouse_pos()
        if data_x is not None:
            updated |= self.pos_label.set_text('%.10f  %.10f' %
                                               (data_x, data_y))

        new_data = None
        with self.data_lock:
            if self.new_data:
                new_data = self.new_data
                self.new_data = []

        if new_data:
            updated = True

            self.t_measurements.append(
                [m._timestamp for m in new_data],
                [m.temp_c for m in new_data])
            self.t_lines.set_x_y_data(self.t_measurements.X,
                                      self.t_measurements.Y)

            self.p_measurements.append(
                [m._timestamp for m in new_data],
                [m.pressure_psi for m in new_data])
            self.p_lines.set_x_y_data(self.p_measurements.X,
                                      self.p_measurements.Y)

            timestamps = []
            pressures  = []
            for i in range(0, math.ceil(self.p_measurements.X[-1]),
                           self.period):
                timestamps.append(i + self.period)
                pressures.append(
                    self.p_measurements.get_avg_value(i, i + self.period))
            self.p_slow_lines.set_x_y_data(timestamps, pressures)

        return updated

    def measurement_callback(self, m):
        with self.data_lock:
            self.new_data.append(m)
            self.mark_dirty()


def measure_thread(x, tw, csv_file):
    t0 = time.time()
    for m in x.yield_measurements():
        t = time.time()
        m._timestamp = dt = t - t0
        tw.measurement_callback(m)

        if csv_file:
            temp_c = m.temp_c if m.temp_c is not None else math.nan
            pressure_psi = (m.pressure_psi if m.pressure_psi is not None
                            else math.nan)
            csv_file.write('%.6f,%.6f,%.2f,%.5f\n' % (t, dt, temp_c,
                                                      pressure_psi))
            csv_file.flush()


def main(args):
    dev = xtalx.p_sensor.find_one_xti(serial_number=args.serial_number)

    if args.csv_file:
        csv_file = open(  # pylint: disable=R1732
            args.csv_file, 'a+', encoding='utf8')

        pos = csv_file.tell()
        if pos != 0:
            csv_file.seek(0)
            if csv_file.read(28) != 'time,dt,temp_c,pressure_psi\n':
                print('%s does not appear to be a pressure sensor log file.' %
                      args.csv_file)
                return
            csv_file.seek(pos)
        else:
            csv_file.write('time,dt,temp_c,pressure_psi\n')
            csv_file.flush()
    else:
        csv_file = None

    x   = xtalx.p_sensor.make(dev)
    tw  = TrackerWindow(x.serial_num, args.averaging_period_secs)
    mt  = threading.Thread(target=measure_thread, args=(x, tw, csv_file))
    mt.start()

    try:
        glotlib.interact()
    except KeyboardInterrupt:
        print()
    finally:
        x.halt_yield()
        mt.join()


def _main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--serial_number', '-s')
    parser.add_argument('--csv-file')
    parser.add_argument('--averaging-period-secs', type=int, default=3)
    args = parser.parse_args()
    main(args)


if __name__ == '__main__':
    _main()



# Copyright (c) 2023 by Phase Advanced Sensor Systems, Inc.
# All rights reserved.
import math

import xtalx.z_sensor
from xtalx.tools.config import Config
from xtalx.tools.influxdb import InfluxDBPushQueue


class SearchParams:
    def __init__(self, name, f0, f1, df):
        self.name = name
        self.f0   = f0
        self.f1   = f1
        self.df   = df


SEARCH_PARAMS = [
    SearchParams('Fluid',  20000, 35000, 50),
    SearchParams('Gas',    29000, 33000, 5),
    SearchParams('Vacuum', 32700, 32900, 1),
]
SEARCH_PARAMS_DICT = {sp.name : sp for sp in SEARCH_PARAMS}


class ZArgs:
    '''
    Helper class to hold common arguments parsed from the command line.
    '''
    def __init__(self, amplitude, f0, f1, df, nfreqs, search_time_secs,
                 sweep_time_secs, settle_ms):
        self.amplitude        = amplitude
        self.f0               = f0
        self.f1               = f1
        self.f_min            = min(f0, f1)
        self.f_max            = max(f0, f1)
        self.df               = df
        self.nfreqs           = nfreqs
        self.search_time_secs = search_time_secs
        self.sweep_time_secs  = sweep_time_secs
        self.settle_ms        = settle_ms


class ZLogger:
    '''
    Helper class that takes care of logging measurements to the console as well
    as writing fit and dump files, if specified.
    '''
    def __init__(self, fit_file, dump_file):
        if dump_file:
            dump_file = open(  # pylint: disable=R1732
                    dump_file, 'a', encoding='utf8')
            dump_file.write('sweep,time_ns,f,dt,zx_real,zx_imag,RR,amplitude\n')
            dump_file.flush()
        else:
            dump_file = None

        if fit_file:
            fit_file = open(  # pylint: disable=R1732
                    fit_file, 'a', encoding='utf8')
            fit_file.write('time_ns,sweep,hires,peak_hz,peak_fwhm,RR,temp_c,'
                           'density_g_per_ml,viscosity_cp,amplitude\n')
            fit_file.flush()
        else:
            fit_file = None

        self.fit_file  = fit_file
        self.dump_file = dump_file

    def log_chirp(self, tc, nchirps, lf):
        if lf is not None:
            tc.log('C', 'i %u peak_hz %.3f peak_fwhm %.3f RR %.6f' %
                   (nchirps - 1, lf.x0, 2 * lf.W, lf.RR))
        else:
            tc.log('c', 'i %u' % nchirps)

    def log_sweep(self, tc, pt, t0_ns, points, fw_fit, hires, temp_freq):
        tag = 'H' if hires else 'S'
        f0  = points[0].f
        f1  = points[-1].f
        if fw_fit is None:
            T, D, V = temp_freq, None, None
            tc.log(tag, 'i %u f0 %f f1 %f T %s D %s V %s' %
                   (pt.sweep, f0, f1, T, D, V))
        else:
            T = fw_fit.temp_c
            D = fw_fit.density_g_per_ml
            V = fw_fit.viscosity_cp
            tc.log(tag, 'i %u f0 %f f1 %f peak_hz %.3f peak_fwhm %.3f S %.2f '
                   'RR %.6f temp_hz %s T %s D %s V %s' %
                   (pt.sweep, f0, f1, fw_fit.peak_hz, fw_fit.peak_fwhm,
                    fw_fit.strength, fw_fit.RR, temp_freq, T, D, V))

        if self.dump_file:
            for p in points:
                self.dump_file.write('%s,%u,%s,%.3f,%s,%s,%s,%u\n' %
                                     (pt.sweep, t0_ns, p.f, p.nbufs / 1000.,
                                      p.z.real, p.z.imag, p.RR[1],
                                      pt.amplitude))
            self.dump_file.flush()

        if self.fit_file:
            self.fit_file.write(
                '%u,%s,%u,%.3f,%.3f,%.6f,%.6f,%.6f,%.6f,%u\n' %
                (t0_ns, pt.sweep,
                 1 if hires else 0,
                 fw_fit.peak_hz if fw_fit else math.nan,
                 fw_fit.peak_fwhm if fw_fit else math.nan,
                 fw_fit.RR if fw_fit is not None else math.nan,
                 T if T is not None else math.nan,
                 D if D is not None else math.nan,
                 V if V is not None else math.nan,
                 pt.amplitude))
            self.fit_file.flush()

        return T, D, V


class ZDelegate(xtalx.z_sensor.peak_tracker.Delegate):
    '''
    Helper delegate class that simply logs sweeps using the ZLogger object.
    '''
    def __init__(self, z_logger):
        self.z_logger = z_logger

    def chirp_callback(self, tc, _pt, n, lf, _X_data, _Y_data, _X_lorentzian,
                       _Y_lorentzian):
        self.z_logger.log_chirp(tc, n, lf)

    def sweep_callback(self, tc, pt, t0_ns, _duration_ms, points, fw_fit,
                       hires, temp_freq):
        self.z_logger.log_sweep(tc, pt, t0_ns, points, fw_fit, hires, temp_freq)


def parse_config(rv):
    if not rv.config:
        return None, None

    with open(rv.config, encoding='utf8') as f:
        c = Config(f.readlines(), ['influx_host', 'influx_user',
                                   'influx_password', 'influx_database'])

        ipq = InfluxDBPushQueue(c.influx_host, 8086, c.influx_user,
                                c.influx_password, database=c.influx_database,
                                ssl=True, verify_ssl=True, timeout=100)

    return c, ipq


def parse_args(tc, rv):
    if rv.freq_0 is not None or rv.freq_1 is not None:
        if rv.freq_0 is None or rv.freq_1 is None:
            raise Exception('Most specify neither or both of --freq-0, '
                            '--freq-1')
        SEARCH_PARAMS.insert(0, SearchParams('%s - %s' % (rv.freq_0, rv.freq_1),
                                             float(rv.freq_0), float(rv.freq_1),
                                             rv.df or 1))
        SEARCH_PARAMS_DICT[SEARCH_PARAMS[0].name] = SEARCH_PARAMS[0]

    if tc.yield_Y:
        tc.info('Tracking admittance.')
    else:
        tc.info('Tracking impedance.')

    amplitude = tc.parse_amplitude(rv.amplitude)

    tc.info('Using amplitude of %u DAC codes.' % amplitude)
    volts = tc.dac_to_a(amplitude)
    if volts is not None:
        tc.info('Using amplitude of %f V.' % volts)

    if not 0 <= amplitude <= 2000:
        raise Exception('Amplitude not in range 0 to 2000.')

    f0 = SEARCH_PARAMS[0].f0
    f1 = SEARCH_PARAMS[0].f1
    df = SEARCH_PARAMS[0].df

    return (ZArgs(amplitude, f0, f1, df, rv.nfreqs, rv.search_time_secs,
                  rv.sweep_time_secs, rv.settle_ms),
            ZLogger(rv.fit_file, rv.dump_file))


def add_arguments(parser):
    parser.add_argument('--amplitude', '-a')
    parser.add_argument('--freq-0', '-0')
    parser.add_argument('--freq-1', '-1')
    parser.add_argument('--df', type=float)
    parser.add_argument('--nfreqs', '-n', type=int, default=100)
    parser.add_argument('--verbose', action='store_true')
    parser.add_argument('--search-time-secs', type=float, default=30)
    parser.add_argument('--sweep-time-secs', type=float, default=30)
    parser.add_argument('--settle-ms', type=int, default=5000)
    parser.add_argument('--dump-file', '-d')
    parser.add_argument('--fit-file', '-f')
    parser.add_argument('--sensor', '-s')
    parser.add_argument('--config', '-c')
    parser.add_argument('--track-impedance', action='store_true')
